#property indicator_chart_window 
#property indicator_buffers 1
#property indicator_plots 1

#include <AIV/Matrices.mqh>

input int InpShift = 0;
input int InpDepth = 500;
input int InpLength = 5;
input string InpSymbols = "EURUSD";

CSets * iSets;
CHelpers * iHelpers;

void OnInit()
{
    iSets = new CSets();
    iHelpers = new CHelpers();

    IndicatorSetString(INDICATOR_SHORTNAME, "Statistics");
    IndicatorSetInteger(INDICATOR_DIGITS, Digits());
}

void OnDeinit(const int reason)
{
    delete(iSets);
    delete(iHelpers);
}

int OnCalculate(
    const int bars,
    const int counted,
    const datetime &time[],
    const double &open[],
    const double& high[],
    const double& low[],
    const double &close[],
    const long &tickVolume[],
    const long &volume[],
    const int &spread[])
{
    getCodes(InpLength);
    return bars;
}

int getCodes(int length)
{
    SSets iSeries[];
    SName iCombinations[];

    int order = iSets.getPairs(iSeries, InpSymbols);
    int bars = iSets.getSourceSets(iSeries, PERIOD_CURRENT, order, InpDepth, InpShift);

    if (bars < 1)
    {
        return 0;
    }

    int codes[];

    ArrayResize(codes, length);

    for (int k = 0; k < order; k++)
    {
        ZeroMemory(codes);
        ArrayResize(iCombinations, k + 1);
        iCombinations[k].mData = iSeries[k].mSymbol.mName;
        
        double point = SymbolInfoDouble(iSeries[k].mSymbol.mName, SYMBOL_POINT);

        do
        {
            string comboChain = NULL;

            for (int i = length - 1; i >= 0; i--)
            {
                comboChain = IntegerToString(codes[i]) + comboChain;
            }

            for (int n = bars - 1; n >= length; n--)
            {
                double pips = 0;
                string comboSymbol = NULL;

                for (int i = 0; i < length; i++)
                {
                    string symbolUnit = "X"; 
                    double range = iSeries[k].mClose[n - i] - iSeries[k].mOpen[n - i];

                    if (range > 0) 
                    {
                        symbolUnit = "1";
                        pips += range;
                    }

                    if (range < 0) 
                    {
                        symbolUnit = "0";
                        pips -= range;
                    }

                    comboSymbol = symbolUnit + comboSymbol;
                }

                if (comboChain == comboSymbol)
                {
                    int index = -1;
                    int count = ArraySize(iCombinations[k].mSeries);

                    for (int i = 0; i < count; i++)
                    {
                        if (iCombinations[k].mSeries[i].mSymbol.mName == comboChain)
                        {
                            index = i;
                            break;
                        }
                    }

                    if (index < 0)
                    {
                        ArrayResize(iCombinations[k].mSeries, count + 1);
                        ZeroMemory(iCombinations[k].mSeries[count]);
                        index = count;
                    }

                    SBet volume = getVolume(iSeries[k], n);

                    iCombinations[k].mSeries[index].mBet.mPlus += volume.mPlus;
                    iCombinations[k].mSeries[index].mBet.mMinus += volume.mMinus;
                    iCombinations[k].mSeries[index].mBet.mCount += volume.mPlus - volume.mMinus;

                    iCombinations[k].mSeries[index].mSymbol.mMean++;
                    iCombinations[k].mSeries[index].mSymbol.mOrder += iCombinations[k].mSeries[index].mSymbol.mMean + n;
                    iCombinations[k].mSeries[index].mSymbol.mUnit += MathAbs(pips / point);
                    iCombinations[k].mSeries[index].mSymbol.mName = comboChain;
                }
            }

        } while (iHelpers.getChain(codes, length));
    }

    string res = "\n";

    for (int k = 0; k < order; k++)
    {
        int count = ArraySize(iCombinations[k].mSeries);

        res += iCombinations[k].mData + "\n";

        for (int n = 0; n < count; n++)
        {
            res += 
                iCombinations[k].mSeries[n].mSymbol.mName + "   #   " + 
                "Count : " + pad(DoubleToString(iCombinations[k].mSeries[n].mSymbol.mMean, 1), 4) + "   #   " + 
                "Weight : " + pad(DoubleToString(iCombinations[k].mSeries[n].mSymbol.mOrder, 1), 5) + "   #   " + 
                "Pips : " + pad(DoubleToString(iCombinations[k].mSeries[n].mSymbol.mUnit, 1), 7) + "   #   " + 
                "Volumes : " + pad(DoubleToString(iCombinations[k].mSeries[n].mBet.mPlus, 1), 8) + "  -  " + 
                pad(DoubleToString(iCombinations[k].mSeries[n].mBet.mMinus, 1), 8) + "  =  " + 
                pad(DoubleToString(iCombinations[k].mSeries[n].mBet.mCount, 1), 8) + "\n";
        }
    }

    Comment(res);

    return 1;
}

SBet getVolume(SSets &iSeries, const int index)
{
    SSets series;

    if (index < 1)
    {
        return series.mBet;
    }

    series.mSymbol.mName = iSeries.mSymbol.mName;

    int seconds = PeriodSeconds(PERIOD_M1);
    int minutes = (int) MathFloor((iSeries.mTime[index] - iSeries.mTime[index - 1]) / seconds);

    iSets.setupSet(series, minutes);

    int count = iSets.getQuotes(series, PERIOD_M1, iSeries.mTime[index], minutes);

    for (int k = 0; k < count; k++)
    {
        double range = series.mClose[k] - series.mOpen[k];
        double value = (double) series.mVolume[k];

        if (range > 0)
        {
            series.mBet.mPlus += MathAbs(value);
        }

        if (range < 0)
        {
            series.mBet.mMinus += MathAbs(value);
        }
    }

    return series.mBet;
}

string pad(const string data, const int length)
{
    string padding = NULL;
    int count = length - StringLen(data);

    for (int k = 0; k < count; k++)
    {
        padding += " ";
    }

    return padding + data; 
}